import icemac.ab.importer.reader.testing
import icemac.ab.importxls.xls


class TestXLS(icemac.ab.importer.reader.testing.ReaderTest):

    reader_class = icemac.ab.importxls.xls.XLSReader
    import_file = 'long.xls'
    import_file_short = 'short.xls'

    def test_getFieldSamples_float_values_are_returned_as_unicode(self):
        self.assertEqual(
            [u'3.1415', u'2.7'],
            list(self.getReader('edge.xls').getFieldSamples(u'float-col')))

    def test_xls__XLSReader____del____1(self):
        """It does not break in destructor if the file is already closed."""
        reader = self.getReader('edge.xls')
        reader.contents.close()
        reader.contents = None
        del reader



from icemac.addressbook.i18n import MessageFactory as _
import datetime
import icemac.ab.importer.reader.base
import mmap
import xlrd


class XLSReader(icemac.ab.importer.reader.base.BaseReader):

    """Import reader for XLS files."""

    title = _(u'Microsoft Excel file')
    contents = None

    @classmethod
    def open(cls, file_handle):
        reader = super(XLSReader, cls).open(file_handle)
        reader.contents = mmap.mmap(
            reader.file.fileno(), 0, access=mmap.ACCESS_READ)
        reader.wb = xlrd.open_workbook(file_contents=reader.contents)
        reader.sheet = reader.wb.sheet_by_index(0)
        reader.field_names = reader.sheet.row_values(0)
        return reader

    def __del__(self):
        if self.contents is not None:
            self.contents.close()
        super(XLSReader, self).__del__()

    def _convert_value(self, value):
        res = value.value
        if value.ctype == xlrd.XL_CELL_DATE:
            date_tuple = xlrd.xldate_as_tuple(res, self.wb.datemode)
            res = datetime.date(*date_tuple[:3])
        elif res == '':
            res = None
        elif isinstance(res, float):
            res = unicode(res)
        return res

    def getFieldNames(self):
        """Get the names of the fields in the file."""
        return self.field_names

    def getFieldSamples(self, field_name):
        """Get sample values for a field."""
        for val in self.sheet.col_slice(self.field_names.index(field_name),
                                        1, 4):
            val = self._convert_value(val)
            if val is None:
                val = u''
            elif isinstance(val, datetime.date):
                val = unicode(val.strftime('%Y-%m-%d'))
            yield val

    def __iter__(self):
        """Iterate over the file."""
        for index in xrange(1, self.sheet.nrows):
            yield [self._convert_value(val) for val in self.sheet.row(index)]



# -*- coding: utf-8 -*-
import os.path
import setuptools


def read(*path_elements):
    """Read a file."""
    return "\n\n" + file(os.path.join(*path_elements)).read()

version = '2.3'
name = 'icemac.ab.importxls'

setuptools.setup(
    name=name,
    version=version,
    description="Import XLS files into icemac.addressbook.",
    long_description=(
        read('README.rst') +
        read('CHANGES.rst')
    ),
    keywords='icemac.addressbook xls excel import',
    author='Michael Howitz',
    author_email='icemac@gmx.net',
    url='https://pypi.org/project/%s' % name,
    license='ZPL 2.1',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Paste',
        'Framework :: Zope3',
        'License :: OSI Approved',
        'License :: OSI Approved :: Zope Public License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 2 :: Only',
    ],
    packages=setuptools.find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['icemac', 'icemac.ab'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'icemac.ab.importer >= 1.0',
        'setuptools',
        'xlrd',
    ],
    extras_require=dict(
        test=['zope.testing >= 3.8.0',
              ]),
)
